==> bucketlib-fabric-1.21.3-4.2.4.0-sources.jar: de/cech12/bucketlib/util/WorldInteractionUtil.java <==
package de.cech12.bucketlib.util;

import de.cech12.bucketlib.api.BucketLibTags;
import de.cech12.bucketlib.api.item.UniversalBucketItem;
import de.cech12.bucketlib.platform.Services;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2275;
import net.minecraft.class_2338;
import net.minecraft.class_2680;
import net.minecraft.class_3218;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import net.minecraft.class_3965;
import net.minecraft.class_5328;

public class WorldInteractionUtil {

    private WorldInteractionUtil() {}

    public static class_1269 tryMilkLivingEntity(class_1799 itemStack, class_1309 entity, class_1657 player, class_1268 interactionHand) {
        if (!entity.method_5864().method_20210(BucketLibTags.EntityTypes.MILKABLE)) {
            return class_1269.field_5811;
        }
        player.method_6122(interactionHand, new class_1799(class_1802.field_8550));
        //deactivate instabuild for the fake interaction to avoid side effects like adding additional filled vanilla buckets into inventory
        boolean previousInstabuildValue = player.method_31549().field_7477;
        player.method_31549().field_7477 = false;
        class_1269 result = player.method_7287(entity, interactionHand);
        player.method_31549().field_7477 = previousInstabuildValue;
        if (result.method_23665()) {
            itemStack = class_5328.method_30012(itemStack.method_7972(), player, BucketLibUtil.addMilk(ItemStackUtil.copyStackWithSize(itemStack, 1)));
        }
        player.method_6122(interactionHand, itemStack);
        return result;
    }

    public static class_1269 tryPickupFromCauldron(class_1937 level, class_1657 player, class_1268 interactionHand, class_3965 blockHitResult) {
        class_1799 itemstack = player.method_5998(interactionHand);
        if (level.method_8608()) {
            return class_1269.field_5811;
        }
        class_2338 hitBlockPos = blockHitResult.method_17777();
        class_2680 hitBlockState = level.method_8320(hitBlockPos);
        class_2248 hitBlock = hitBlockState.method_26204();
        if (hitBlock instanceof class_2275 cauldronBlock
                && itemstack.method_7909() instanceof UniversalBucketItem bucketItem
                && BucketLibUtil.isEmpty(itemstack)) {
            //check if bucket can hold cauldron content
            if (cauldronBlock == class_2246.field_27098 && bucketItem.canHoldFluid(class_3612.field_15908)
                    || cauldronBlock == class_2246.field_27097 && bucketItem.canHoldFluid(class_3612.field_15910)
                    || cauldronBlock == class_2246.field_27878 && bucketItem.canHoldBlock(class_2246.field_27879)) {
                //fake vanilla bucket using on cauldron
                class_1799 stack = new class_1799(class_1802.field_8550);
                player.method_6122(interactionHand, stack);
                //deactivate instabuild for the fake interaction to avoid side effects like adding additional filled vanilla buckets into inventory
                boolean previousInstabuildValue = player.method_31549().field_7477;
                player.method_31549().field_7477 = false;
                class_1269 interactionResult = hitBlockState.method_55780(stack, level, player, interactionHand, blockHitResult);
                player.method_31549().field_7477 = previousInstabuildValue;
                class_1799 resultItemStack = player.method_5998(interactionHand);
                player.method_6122(interactionHand, itemstack);
                if (interactionResult.method_23665()) {
                    if (resultItemStack.method_7909() == class_1802.field_27876) {
                        return class_1269.field_5812.method_61393(class_5328.method_30012(itemstack, player, BucketLibUtil.addBlock(ItemStackUtil.copyStackWithSize(itemstack, 1), class_2246.field_27879)));
                    } else {
                        return class_1269.field_5812.method_61393(class_5328.method_30012(itemstack, player, BucketLibUtil.addFluid(ItemStackUtil.copyStackWithSize(itemstack, 1), Services.FLUID.getContainedFluid(resultItemStack))));
                    }
                }
            }
        }
        return class_1269.field_5811;
    }

    public static class_1269 tryPlaceIntoCauldron(class_1937 level, class_1657 player, class_1268 interactionHand, class_3965 blockHitResult) {
        class_1799 itemstack = player.method_5998(interactionHand);
        class_2338 hitBlockPos = blockHitResult.method_17777();
        class_2680 hitBlockState = level.method_8320(hitBlockPos);
        class_2248 hitBlock = hitBlockState.method_26204();
        if (hitBlock instanceof class_2275 cauldronBlock
                && itemstack.method_7909() instanceof UniversalBucketItem
                && cauldronBlock == class_2246.field_10593
                && !BucketLibUtil.containsEntityType(itemstack)) {
            //fake vanilla bucket using on cauldron
            class_3611 bucketFluid = BucketLibUtil.getFluid(itemstack);
            class_2248 bucketBlock = BucketLibUtil.getBlock(itemstack);
            class_3218 serverLevel = (level instanceof class_3218) ? (class_3218) level : null;
            if (bucketFluid == class_3612.field_15908 || bucketFluid == class_3612.field_15910) {
                class_1799 stack = new class_1799(bucketFluid.method_15774());
                player.method_6122(interactionHand, stack);
                //deactivate instabuild for the fake interaction to avoid side effects like adding additional empty vanilla buckets into inventory
                boolean previousInstabuildValue = player.method_31549().field_7477;
                player.method_31549().field_7477 = false;
                class_1269 interactionResult = hitBlockState.method_55780(stack, level, player, interactionHand, blockHitResult);
                player.method_31549().field_7477 = previousInstabuildValue;
                player.method_6122(interactionHand, itemstack);
                if (interactionResult.method_23665()) {
                    return class_1269.field_5812.method_61393(BucketLibUtil.createEmptyResult(itemstack, player, BucketLibUtil.removeFluid(itemstack, serverLevel, player), interactionHand, true));
                }
            } else if (bucketBlock == class_2246.field_27879) {
                class_1799 stack = new class_1799(class_1802.field_27876);
                player.method_6122(interactionHand, stack);
                //deactivate instabuild for the fake interaction to avoid side effects like adding additional empty vanilla buckets into inventory
                boolean previousInstabuildValue = player.method_31549().field_7477;
                player.method_31549().field_7477 = false;
                class_1269 interactionResult = hitBlockState.method_55780(stack, level, player, interactionHand, blockHitResult);
                player.method_31549().field_7477 = previousInstabuildValue;
                player.method_6122(interactionHand, itemstack);
                if (interactionResult.method_23665()) {
                    return class_1269.field_5812.method_61393(BucketLibUtil.createEmptyResult(itemstack, player, BucketLibUtil.removeBlock(itemstack, serverLevel, player, true), interactionHand, true));
                }
            }
        }
        return class_1269.field_5811;
    }

}

==> bucketlib-fabric-1.21-4.1.6.0-sources.jar: de/cech12/bucketlib/util/WorldInteractionUtil.java <==
package de.cech12.bucketlib.util;

import de.cech12.bucketlib.api.BucketLibTags;
import de.cech12.bucketlib.api.item.UniversalBucketItem;
import de.cech12.bucketlib.platform.Services;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1271;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2275;
import net.minecraft.class_2338;
import net.minecraft.class_2680;
import net.minecraft.class_3218;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import net.minecraft.class_3965;
import net.minecraft.class_5328;
import net.minecraft.class_9062;

public class WorldInteractionUtil {

    private WorldInteractionUtil() {}

    public static class_1269 tryMilkLivingEntity(class_1799 itemStack, class_1309 entity, class_1657 player, class_1268 interactionHand) {
        if (!entity.method_5864().method_20210(BucketLibTags.EntityTypes.MILKABLE)) {
            return class_1269.field_5811;
        }
        player.method_6122(interactionHand, new class_1799(class_1802.field_8550));
        //deactivate instabuild for the fake interaction to avoid side effects like adding additional filled vanilla buckets into inventory
        boolean previousInstabuildValue = player.method_31549().field_7477;
        player.method_31549().field_7477 = false;
        class_1269 result = player.method_7287(entity, interactionHand);
        player.method_31549().field_7477 = previousInstabuildValue;
        if (result.method_23665()) {
            itemStack = class_5328.method_30012(itemStack.method_7972(), player, BucketLibUtil.addMilk(ItemStackUtil.copyStackWithSize(itemStack, 1)));
        }
        player.method_6122(interactionHand, itemStack);
        return result;
    }

    public static class_1271<class_1799> tryPickupFromCauldron(class_1937 level, class_1657 player, class_1268 interactionHand, class_3965 blockHitResult) {
        class_1799 itemstack = player.method_5998(interactionHand);
        if (level.method_8608()) {
            return class_1271.method_22430(itemstack);
        }
        class_2338 hitBlockPos = blockHitResult.method_17777();
        class_2680 hitBlockState = level.method_8320(hitBlockPos);
        class_2248 hitBlock = hitBlockState.method_26204();
        if (hitBlock instanceof class_2275 cauldronBlock
                && itemstack.method_7909() instanceof UniversalBucketItem bucketItem
                && BucketLibUtil.isEmpty(itemstack)) {
            //check if bucket can hold cauldron content
            if (cauldronBlock == class_2246.field_27098 && bucketItem.canHoldFluid(class_3612.field_15908)
                    || cauldronBlock == class_2246.field_27097 && bucketItem.canHoldFluid(class_3612.field_15910)
                    || cauldronBlock == class_2246.field_27878 && bucketItem.canHoldBlock(class_2246.field_27879)) {
                //fake vanilla bucket using on cauldron
                class_1799 stack = new class_1799(class_1802.field_8550);
                player.method_6122(interactionHand, stack);
                //deactivate instabuild for the fake interaction to avoid side effects like adding additional filled vanilla buckets into inventory
                boolean previousInstabuildValue = player.method_31549().field_7477;
                player.method_31549().field_7477 = false;
                class_9062 interactionResult = hitBlockState.method_55780(stack, level, player, interactionHand, blockHitResult);
                player.method_31549().field_7477 = previousInstabuildValue;
                class_1799 resultItemStack = player.method_5998(interactionHand);
                player.method_6122(interactionHand, itemstack);
                if (interactionResult.method_55643()) {
                    if (resultItemStack.method_7909() == class_1802.field_27876) {
                        return new class_1271<>(interactionResult.method_55645(), class_5328.method_30012(itemstack, player, BucketLibUtil.addBlock(ItemStackUtil.copyStackWithSize(itemstack, 1), class_2246.field_27879)));
                    } else {
                        return new class_1271<>(interactionResult.method_55645(), class_5328.method_30012(itemstack, player, BucketLibUtil.addFluid(ItemStackUtil.copyStackWithSize(itemstack, 1), Services.FLUID.getContainedFluid(resultItemStack))));
                    }
                }
            }
        }
        return class_1271.method_22430(itemstack);
    }

    public static class_1271<class_1799> tryPlaceIntoCauldron(class_1937 level, class_1657 player, class_1268 interactionHand, class_3965 blockHitResult) {
        class_1799 itemstack = player.method_5998(interactionHand);
        class_2338 hitBlockPos = blockHitResult.method_17777();
        class_2680 hitBlockState = level.method_8320(hitBlockPos);
        class_2248 hitBlock = hitBlockState.method_26204();
        if (hitBlock instanceof class_2275 cauldronBlock
                && itemstack.method_7909() instanceof UniversalBucketItem
                && cauldronBlock == class_2246.field_10593
                && !BucketLibUtil.containsEntityType(itemstack)) {
            //fake vanilla bucket using on cauldron
            class_3611 bucketFluid = BucketLibUtil.getFluid(itemstack);
            class_2248 bucketBlock = BucketLibUtil.getBlock(itemstack);
            class_3218 serverLevel = (level instanceof class_3218) ? (class_3218) level : null;
            if (bucketFluid == class_3612.field_15908 || bucketFluid == class_3612.field_15910) {
                class_1799 stack = new class_1799(bucketFluid.method_15774());
                player.method_6122(interactionHand, stack);
                //deactivate instabuild for the fake interaction to avoid side effects like adding additional empty vanilla buckets into inventory
                boolean previousInstabuildValue = player.method_31549().field_7477;
                player.method_31549().field_7477 = false;
                class_9062 interactionResult = hitBlockState.method_55780(stack, level, player, interactionHand, blockHitResult);
                player.method_31549().field_7477 = previousInstabuildValue;
                player.method_6122(interactionHand, itemstack);
                if (interactionResult.method_55643()) {
                    return new class_1271<>(interactionResult.method_55645(), BucketLibUtil.createEmptyResult(itemstack, player, BucketLibUtil.removeFluid(itemstack, serverLevel, player), interactionHand, true));
                }
            } else if (bucketBlock == class_2246.field_27879) {
                class_1799 stack = new class_1799(class_1802.field_27876);
                player.method_6122(interactionHand, stack);
                //deactivate instabuild for the fake interaction to avoid side effects like adding additional empty vanilla buckets into inventory
                boolean previousInstabuildValue = player.method_31549().field_7477;
                player.method_31549().field_7477 = false;
                class_9062 interactionResult = hitBlockState.method_55780(stack, level, player, interactionHand, blockHitResult);
                player.method_31549().field_7477 = previousInstabuildValue;
                player.method_6122(interactionHand, itemstack);
                if (interactionResult.method_55643()) {
                    return new class_1271<>(interactionResult.method_55645(), BucketLibUtil.createEmptyResult(itemstack, player, BucketLibUtil.removeBlock(itemstack, serverLevel, player, true), interactionHand, true));
                }
            }
        }
        return class_1271.method_22430(itemstack);
    }

}
